
#The following is the script used to simulate the topology described in
#Section 4.2 Rerouting Prioritized Packets upon Failures:
#=========================
# Create simulator object
#=========================
set ns [new Simulator]
#===================================================
# Open files to write trace-data for NAM and Xgraph
#===================================================
set nf [open mpls.nam w]
$ns namtrace-all $nf
set f0 [open mpls.tr w]
$ns trace-all $f0
#=======================================================================
# Finish procedure which closes the trace file and opens Xgraph and NAM
#=======================================================================
proc finish {} {
global ns nf f0
$ns flush-trace
close $nf
close $f0
exec ../../nam/nam mpls.nam &
exit 0
}
#==============================================
# Set dynamic distance-vector routing protocol
#==============================================
$ns rtproto DV
#==========================================================
# Define trigger strategy, Label Distribution Control Mode
# and Label Allocation and Distribution Scheme
#==========================================================
Classifier/Addr/MPLS set control_driven_ 1
#
# Turn on all traces to stdout
#
#Agent/LDP set trace_ldp_ 1
#Classifier/Addr/MPLS set trace_mpls_ 1
#
# use 'List' scheduling of events
#
$ns use-scheduler List
#==============================================================
# define nodes and MPLS LSRs (in case of a LSR, the [$ns node]
# command has to be preceded by node-config -MPLS ON
# and succeeded by node-config -MPLS OFF
#==============================================================
set node0 [$ns node]
set node1 [$ns node]
$ns node-config -MPLS ON
set LSR2 [$ns node]
set LSR3 [$ns node]
set LSR4 [$ns node]
set LSR5 [$ns node]
set LSR6 [$ns node]
set LSR7 [$ns node]
set LSR8 [$ns node]
set LSR9 [$ns node]
$ns node-config -MPLS OFF
set node10 [$ns node]
set node11 [$ns node]
#====================================================================
# Define links, bandwidth 1 Mb, delay 10ms, queue managementDropTail
#====================================================================
$ns duplex-link $node0 $LSR2 1Mb 10ms DropTail
$ns duplex-link $node1 $LSR2 1Mb 10ms DropTail
$ns duplex-link $LSR2 $LSR3 1Mb 10ms DropTail
$ns duplex-link $LSR3 $LSR4 1Mb 10ms DropTail
$ns duplex-link $LSR4 $LSR9 1Mb 10ms DropTail
$ns duplex-link $LSR2 $LSR5 1Mb 10ms DropTail
$ns duplex-link $LSR5 $LSR6 1Mb 10ms DropTail
$ns duplex-link $LSR6 $LSR9 1Mb 10ms DropTail
$ns duplex-link $LSR2 $LSR7 1Mb 10ms DropTail
$ns duplex-link $LSR7 $LSR8 0.5Mb 10ms DropTail
$ns duplex-link $LSR8 $LSR9 1Mb 10ms DropTail
$ns duplex-link $LSR9 $node10 1Mb 10ms DropTail
$ns duplex-link $LSR9 $node11 1Mb 10ms DropTail
#===============================
# Control layout of the network
#===============================
$ns duplex-link-op $node0 $LSR2 orient right-down
$ns duplex-link-op $node1 $LSR2 orient right-up
$ns duplex-link-op $LSR2 $LSR3 orient right-up
$ns duplex-link-op $LSR3 $LSR4 orient right
$ns duplex-link-op $LSR4 $LSR9 orient right-down
$ns duplex-link-op $LSR2 $LSR5 orient right-down
$ns duplex-link-op $LSR5 $LSR6 orient right
$ns duplex-link-op $LSR6 $LSR9 orient right-up
$ns duplex-link-op $LSR2 $LSR7 orient right
$ns duplex-link-op $LSR7 $LSR8 orient right
$ns duplex-link-op $LSR8 $LSR9 orient right
$ns duplex-link-op $LSR9 $node10 orient right-up
$ns duplex-link-op $LSR9 $node11 orient right-down
#===================================================================
# The default value of a link cost (1) can be adjusted
# Notice that the procedure sets the cost along one direction only!
#===================================================================
$ns cost $LSR3 $LSR4 3
$ns cost $LSR4 $LSR3 3
$ns cost $LSR5 $LSR6 3
$ns cost $LSR6 $LSR5 3
#=========================================================
# Install/configure LDP agents on all MPLS nodes,
# and set path restoration function that reroutes traffic
# around a link failure in a LSP to an alternative LSP.
# There are 2 options as follows:
# "new": create new alternative path if one doesn't exist
# "drop": do not create any new alternative path
#
# Adjust loop length to address all LSRs (MPLS nodes).
#=========================================================
for {set i 2} {$i < 10} {incr i} {
set a LSR$i
for {set j [expr $i+1]} {$j < 10} {incr j} {
set b LSR$j
eval $ns LDP-peer $$a $$b
}
set m [eval $$a get-module "MPLS"]
$m enable-reroute "drop"
}
#==============================
# Set ldp-message color in NAM
#==============================
$ns ldp-request-color blue
$ns ldp-mapping-color red
$ns ldp-withdraw-color magenta
$ns ldp-release-color orange
$ns ldp-notification-color yellow
#
# Define procedure to create a CBR traffic flow and connect it to a UDP agent
#
proc attach-expoo-traffic { node sink size burst idle rate} {
global ns
set udp [new Agent/UDP]
$ns attach-agent $node $udp
set traffic [new Application/Traffic/Exponential]
$traffic set packetSize_ $size
$traffic set burst_time_ $burst
$traffic set idle_time_ $idle
$traffic set rate_ $rate
$traffic attach-agent $udp
$ns connect $udp $sink
return $traffic
}
proc attach-expoo-traffic-with-fixed-prio { node sink size burst idle rate prio}
{
global ns
set udp [new Agent/UDP]
$udp set-fixed-prio $prio
$ns attach-agent $node $udp
set traffic [new Application/Traffic/Exponential]
$traffic set packetSize_ $size
$traffic set burst_time_ $burst
$traffic set idle_time_ $idle
$traffic set rate_ $rate
$traffic attach-agent $udp
$ns connect $udp $sink
return $traffic
}
proc attach-expoo-traffic-with-uniform-prio { node sink size burst idle rate} {
global ns
set udp [new Agent/UDP]
$udp set-uniform-prio
$ns attach-agent $node $udp
set traffic [new Application/Traffic/Exponential]
$traffic set packetSize_ $size
$traffic set burst_time_ $burst
$traffic set idle_time_ $idle
$traffic set rate_ $rate
$traffic attach-agent $udp
$ns connect $udp $sink
return $traffic
}
#
# Create a traffic sink and attach it to the node node8
#
set sink0 [new Agent/LossMonitor]
$ns attach-agent $node10 $sink0
set sink1 [new Agent/LossMonitor]
$ns attach-agent $node11 $sink1
#
# Create a traffic source
#
set src0 [attach-expoo-traffic-with-fixed-prio $node1 $sink0 200 0 0 400k 1]
set src1 [attach-expoo-traffic-with-fixed-prio $node1 $sink0 200 0 0 400k 2]
$ns at 0.10 "[$LSR9 get-module MPLS] ldp-trigger-by-withdraw 10 -1"
$ns at 0.30 "[$LSR2 get-module MPLS] make-explicit-route 9 2_3_4_9 1000 -1"
$ns at 0.30 "[$LSR2 get-module MPLS] make-explicit-route 9 2_5_6_9 1001 -1"
$ns at 0.50 "[$LSR2 get-module MPLS] flow-erlsp-install 10 -1 1000 1_2"
$ns at 0.60 "[$LSR2 get-module MPLS] reroute-binding 10 -1 1001 2"
$ns at 0.70 "$src0 start"
$ns at 0.70 "$src1 start"
$ns rtmodel-at 1.00 down $LSR2 $LSR3
$ns rtmodel-at 1.50 up $LSR3 $LSR2
$ns at 2.50 "$src0 stop"
$ns at 2.50 "$src1 stop"
$ns at 3.00 "finish"
#
# The last line finally starts the simulation
#
$ns run